#include <jni.h>
#include <string.h>
#include <unistd.h>
#include <speex/speex.h>
#include <android/log.h>
#include <speex/speex_echo.h>
#include <speex/speex_preprocess.h>

#define LOGI(...) __android_log_print(ANDROID_LOG_INFO,"lee",__VA_ARGS__)

static int codec_open = 0;
static int dec_frame_size;
static int enc_frame_size;
static SpeexBits ebits, dbits;
static SpeexPreprocessState * m_st;
//static SpeexEchoState *echo_state;
void *enc_state;
void *dec_state;

extern "C" JNIEXPORT jint JNICALL Java_com_lee_speex_jni_Speex_open(JNIEnv *env,
		jobject obj, jint compression) {
	int tmp;
	if (codec_open++ != 0)
		return (jint) 0;
	speex_bits_init(&ebits);
	speex_bits_init(&dbits);
	enc_state = speex_encoder_init(&speex_nb_mode);
	dec_state = speex_decoder_init(&speex_nb_mode);
	tmp = compression;
	speex_encoder_ctl(enc_state, SPEEX_SET_QUALITY, &tmp);
	speex_encoder_ctl(enc_state, SPEEX_GET_FRAME_SIZE, &enc_frame_size);
	speex_decoder_ctl(dec_state, SPEEX_GET_FRAME_SIZE, &dec_frame_size);
	return (jint) 0;
}

extern "C" JNIEXPORT jint JNICALL Java_com_lee_speex_jni_Speex_preprocess(
		JNIEnv *env, jobject obj, jint sample_rate) {
	m_st = speex_preprocess_state_init(enc_frame_size, sample_rate);
	int denoise = 1;
	int noiseSuppress = -25;
	speex_preprocess_ctl(m_st, SPEEX_PREPROCESS_SET_DENOISE, &denoise); //降噪
	speex_preprocess_ctl(m_st, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS,
			&noiseSuppress); //设置噪声的dB
	int vad = 1;
	int vadProbStart = 80;
	int vadProbContinue = 65;
	speex_preprocess_ctl(m_st, SPEEX_PREPROCESS_SET_VAD, &vad); //静音检测
	//Set probability required for the VAD to go from silence to voice
	speex_preprocess_ctl(m_st, SPEEX_PREPROCESS_SET_PROB_START, &vadProbStart);
	speex_preprocess_ctl(m_st, SPEEX_PREPROCESS_SET_PROB_CONTINUE,
			&vadProbContinue);
	return 1;
}

extern "C" JNIEXPORT jint JNICALL Java_com_lee_speex_jni_Speex_encode(
		JNIEnv *env, jobject obj, jshortArray lin, jbyteArray encoded) {
	jshort in_enc_tmp[enc_frame_size];
	jbyte out_enc_tmp[enc_frame_size];
	int tot_bytes = 0;
	if (!codec_open)
		return 0;
	speex_bits_reset(&ebits);
	env->GetShortArrayRegion(lin, 0, enc_frame_size, in_enc_tmp);
	if (m_st != 0)
		speex_preprocess_run(m_st, in_enc_tmp);
	speex_encode_int(enc_state, (short *) in_enc_tmp, &ebits);
	tot_bytes = speex_bits_write(&ebits, (char *) out_enc_tmp, enc_frame_size);
	env->SetByteArrayRegion(encoded, 0, tot_bytes, out_enc_tmp);
	return (jint) tot_bytes;
}

extern "C" JNIEXPORT jint JNICALL Java_com_lee_speex_jni_Speex_decode(
		JNIEnv *env, jobject obj, jbyteArray encoded, jshortArray lin,
		jint size) {
	jbyte in_enc_tmp[enc_frame_size];
	jshort out_enc_tmp[dec_frame_size];
	if (!codec_open)
		return 0;
	env->GetByteArrayRegion(encoded, 0, size, in_enc_tmp);
	speex_bits_read_from(&dbits, (char *) in_enc_tmp, size);
	speex_decode_int(dec_state, &dbits, (short *) out_enc_tmp);
	env->SetShortArrayRegion(lin, 0, dec_frame_size, out_enc_tmp);
	return (jint) dec_frame_size;
}

extern "C" JNIEXPORT void JNICALL Java_com_lee_speex_jni_Speex_close(
		JNIEnv *env, jobject obj) {
	if (--codec_open != 0)
		return;
	speex_bits_destroy(&ebits);
	speex_bits_destroy(&dbits);
	speex_decoder_destroy(dec_state);
	speex_encoder_destroy(enc_state);
}

