package com.lee.call.ui;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.TabActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.os.Vibrator;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.widget.ImageButton;
import android.widget.TabHost;
import android.widget.TabHost.OnTabChangeListener;
import android.widget.TabWidget;
import android.widget.TextView;
import android.widget.Toast;

import com.lee.call.custom.ICallContactLayout;
import com.lee.call.custom.ICallDialLayout;
import com.lee.call.custom.ICallLogLayout;
import com.lee.call.custom.ICallMessageLayout;
import com.lee.call.tools.ICallTool;
import com.lee.call.work.WorkManager;

public class ICallTab extends TabActivity implements OnTabChangeListener,
		OnClickListener, OnTouchListener {

	protected static final String[] tags = { "Dial", "Log", "Contact", "Mesage" };
	protected static final String[] tags_cn = { "拨号", "日志", "联系人", "信息" };
	public static final byte t_dial = 0;
	public static final byte t_log = 1;
	public static final byte t_contact = 2;
	public static final byte t_message = 3;
	protected ICallDialLayout dialLayout;
	protected ICallLogLayout logLayout;
	protected ICallContactLayout contactLayout;
	protected ICallMessageLayout messageLayout;
	protected TextView tip_number;

	protected ImageButton action_domain;
	protected ImageButton action_call;
	protected ImageButton action_delete;
	protected ImageButton[] call_buttons;

	protected WorkManager manager;
	protected Vibrator vibrator;
	// 声音产生器
	protected ToneGenerator mToneGenerator;
	protected AudioManager audioManager;
	protected boolean dial_sound;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);
		TabHost tabHost = getTabHost();
		tabHost.setOnTabChangedListener(this);
		vibrator = (Vibrator) getSystemService(VIBRATOR_SERVICE);
		audioManager = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
		mToneGenerator = new ToneGenerator(AudioManager.STREAM_DTMF, 100); // 设置声音的大小
		setVolumeControlStream(AudioManager.STREAM_DTMF);

		dialLayout = new ICallDialLayout(this, R.layout.dial);
		logLayout = new ICallLogLayout(this, R.layout.log);
		contactLayout = new ICallContactLayout(this, R.layout.contact);
		messageLayout = new ICallMessageLayout(this, R.layout.message);
		call_buttons = new ImageButton[12];
		for (int i = 0; i < call_buttons.length; i++) {
			if (i == 9)
				call_buttons[i] = (ImageButton) dialLayout
						.findViewById(R.id.no_star);
			else if (i == 10)
				call_buttons[i] = (ImageButton) dialLayout
						.findViewById(R.id.no_0);
			else if (i == 11)
				call_buttons[i] = (ImageButton) dialLayout
						.findViewById(R.id.no_pound);
			else
				call_buttons[i] = (ImageButton) dialLayout
						.findViewById(R.id.no_1 + i);
			call_buttons[i].setOnTouchListener(this);
		}
		tip_number = (TextView) dialLayout.findViewById(R.id.tip_number);
		action_domain = (ImageButton) dialLayout
				.findViewById(R.id.action_domain);
		action_call = (ImageButton) dialLayout.findViewById(R.id.action_call);
		action_delete = (ImageButton) dialLayout
				.findViewById(R.id.action_delete);
		if (getIntent().getData() != null) {
			String number = getIntent().getData().toString();
			tip_number.setText(number);
		}
		tip_number.addTextChangedListener(new TextWatcher() {

			@Override
			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
				// TODO Auto-generated method stub
			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
				// TODO Auto-generated method stub
			}

			@Override
			public void afterTextChanged(Editable s) {
				// TODO Auto-generated method stub
				if (tip_number.getText().length() > 0)
					tip_number.setSelected(true);
				else
					tip_number.setSelected(false);
			}
		});
		Drawable d_dial = getResources().getDrawable(
				R.drawable.ic_tab_unselected_dialer);
		Drawable d_log = getResources().getDrawable(
				R.drawable.ic_tab_unselected_log);
		Drawable d_contact = getResources().getDrawable(
				R.drawable.ic_tab_unselected_contacts);
		Drawable d_msg = getResources().getDrawable(
				R.drawable.ic_tab_unselected_messages);
		tabHost.addTab(tabHost.newTabSpec(tags[t_dial])
				.setIndicator(tags_cn[t_dial], d_dial).setContent(dialLayout));
		tabHost.addTab(tabHost.newTabSpec(tags[t_log])
				.setIndicator(tags_cn[t_log], d_log).setContent(logLayout));
		tabHost.addTab(tabHost.newTabSpec(tags[t_contact])
				.setIndicator(tags_cn[t_contact], d_contact)
				.setContent(contactLayout));
		tabHost.addTab(tabHost.newTabSpec(tags[t_message])
				.setIndicator(tags_cn[t_message], d_msg)
				.setContent(messageLayout));
		final TabWidget tabWidget = tabHost.getTabWidget();
		for (int i = 0; i < tabWidget.getChildCount(); i++)
			tabWidget.getChildAt(i).getLayoutParams().height = (int) (66 * 1.2);
		String ip = ICallTool.getWifiIP(this);
		if (ip != null) {
			ICallTool.copy(this, ip);
			showMsg("wifi的ip地址成功复制到粘贴板!");
		} else if ((ip = ICallTool.getGprsIp()) != null) {
			ICallTool.copy(this, ip);
			showMsg("gprs的ip地址成功复制到粘贴板!");
		} else
			showMsg("对不起，你的网络不可用!");
		dial_sound = ICallSetting.getDialSound(this);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// TODO Auto-generated method stub
		getMenuInflater().inflate(R.menu.menu, menu);
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// TODO Auto-generated method stub
		switch (item.getItemId()) {
		case R.id.setting:
			Intent intent = new Intent(this, ICallSetting.class);
			startActivityForResult(intent, 0);
			break;
		case R.id.help:
			showDialog(0);
			break;
		case R.id.exit:
			finish();
			break;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		// TODO Auto-generated method stub
		super.onActivityResult(requestCode, resultCode, data);
		dial_sound = ICallSetting.getDialSound(this);
	}

	protected void showMsg(String text) {
		Toast.makeText(this, text, Toast.LENGTH_LONG).show();
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		// TODO Auto-generated method stub
		return new AlertDialog.Builder(this).setTitle("帮助")
				.setMessage("有她，就够啦！")
				.setIcon(android.R.drawable.ic_dialog_info)
				.setNegativeButton("谢谢", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
					}
				}).create();
	}

	@Override
	public void onTabChanged(String arg0) {
		// TODO Auto-generated method stub

	}

	private void setTip(int v) {
		StringBuffer buffer = new StringBuffer();
		if (tip_number.getText().length() > 0)
			buffer.append(tip_number.getText());
		if (v == 42)
			buffer.append('*');
		else if (v == 35)
			buffer.append('#');
		else
			buffer.append(v);
		tip_number.setText(buffer.toString());
	}

	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		int id = v.getId();
		switch (id) {
		case R.id.no_1:
			setTip(1);
			break;
		case R.id.no_2:
			setTip(2);
			break;
		case R.id.no_3:
			setTip(3);
			break;
		case R.id.no_4:
			setTip(4);
			break;
		case R.id.no_5:
			setTip(5);
			break;
		case R.id.no_6:
			setTip(6);
			break;
		case R.id.no_7:
			setTip(7);
			break;
		case R.id.no_8:
			setTip(8);
			break;
		case R.id.no_9:
			setTip(9);
			break;
		case R.id.no_0:
			setTip(0);
			break;
		case R.id.no_star:
			setTip('*');
			break;
		case R.id.no_pound:
			setTip('#');
			break;
		case R.id.action_domain:

			break;
		case R.id.action_call:
			if (manager == null && ICallSetting.getIp(this) != null)
				manager = new WorkManager(this, ICallSetting.getIp(this));
			else
				showMsg("还没有设置ip");
			break;
		case R.id.action_delete:
			int length = tip_number.getText().length();
			if (length > 0)
				tip_number.setText(tip_number.getText().subSequence(0,
						length - 1));
			break;
		}
	}

	@Override
	public void finish() {
		// TODO Auto-generated method stub
		super.finish();
		System.exit(0);
	}

	@Override
	public boolean onTouch(View v, MotionEvent event) {
		// TODO Auto-generated method stub
		if (event.getAction() == MotionEvent.ACTION_DOWN) {
			vibrator.vibrate(33);
			int id = v.getId();
			switch (id) {
			case R.id.no_1:
				playTone(ToneGenerator.TONE_DTMF_1);
				break;
			case R.id.no_2:
				playTone(ToneGenerator.TONE_DTMF_2);
				break;
			case R.id.no_3:
				playTone(ToneGenerator.TONE_DTMF_3);
				break;
			case R.id.no_4:
				playTone(ToneGenerator.TONE_DTMF_4);
				break;
			case R.id.no_5:
				playTone(ToneGenerator.TONE_DTMF_5);
				break;
			case R.id.no_6:
				playTone(ToneGenerator.TONE_DTMF_6);
				break;
			case R.id.no_7:
				playTone(ToneGenerator.TONE_DTMF_7);
				break;
			case R.id.no_8:
				playTone(ToneGenerator.TONE_DTMF_8);
				break;
			case R.id.no_9:
				playTone(ToneGenerator.TONE_DTMF_9);
				break;
			case R.id.no_0:
				playTone(ToneGenerator.TONE_DTMF_0);
				break;
			case R.id.no_star:
				playTone(ToneGenerator.TONE_DTMF_S);
				break;
			case R.id.no_pound:
				playTone(ToneGenerator.TONE_DTMF_P);
				break;
			case R.id.action_domain:
				playTone(ToneGenerator.TONE_DTMF_7);
				break;
			case R.id.action_call:
				playTone(ToneGenerator.TONE_DTMF_8);
				break;
			case R.id.action_delete:
				playTone(ToneGenerator.TONE_DTMF_9);
				break;
			}
		}
		return false;
	}

	/**
	 * 播放按键声音
	 */
	private void playTone(int tone) {
		int ringerMode = audioManager.getRingerMode();
		if (!dial_sound || ringerMode == AudioManager.RINGER_MODE_SILENT
				|| ringerMode == AudioManager.RINGER_MODE_VIBRATE) {
			// 静音或者震动时不发出声音
			return;
		}
		mToneGenerator.startTone(tone, 100); // 发出声音
	}

}
